@@ -1,4 +1,4 @@
-/* rexima 1.2 - a curses-based (and command-line) mixer for Linux.
+/* rexima 1.3 - a curses-based (and command-line) mixer for Linux.
  * Copyright (C) 1996-2002 Russell Marks.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 #include <sys/soundcard.h>
 
 
-#define REXIMA_VER	"1.2"
+#define REXIMA_VER	"1.3"
 
 
 #define DEV_Y_START	3
@@ -42,6 +42,9 @@ char dev_names [SOUND_MIXER_NRDEVICES][80]=SOUND_DEVICE_NAMES;
 int dev_line[SOUND_MIXER_NRDEVICES];
 
 
+char *mixerdev="/dev/mixer";
+
+
 void die(char *str)
 {
 fprintf(stderr,"rexima: couldn't %s.\n",str);
@@ -52,7 +55,7 @@ exit(1);
 void init(int *mixfd,int *existmask,int *canrecmask,int *isrecmask,
           int *stereomask)
 {
-if(((*mixfd)=open("/dev/mixer",O_RDWR))<0) die("open /dev/mixer");
+if(((*mixfd)=open(mixerdev,O_RDWR))<0) die("open mixer device");
 if(ioctl(*mixfd,SOUND_MIXER_READ_DEVMASK,existmask)==-1) die("ioctl");
 if(ioctl(*mixfd,SOUND_MIXER_READ_RECMASK,canrecmask)==-1) die("ioctl");
 if(ioctl(*mixfd,SOUND_MIXER_READ_RECSRC,isrecmask)==-1) die("ioctl");
@@ -221,13 +224,16 @@ int f,count;
 puts(
 "rexima " REXIMA_VER " - copyright (c) 1996-2002 Russell Marks.\n"
 "\n"
-"usage: rexima [-hv] [device <level | offset | left,right | <rec | norec>>\n"
+"usage: rexima [-hv] [-d mixer_device_file]\n"
+"		     [device <level | offset | left,right | <rec | norec>>\n"
 "                     [device ...]]\n"
 "\n"
-"	-h	this usage help.\n"
-"	-v	show current mixer settings.\n"
-"	device	a device to set the level of. For this soundcard, /dev/mixer");
-printf("\t\tallows the levels of these devices to be set:\n\n\t\t");
+"	-d	specify mixer device file to use (ordinarily /dev/mixer).\n"
+"	-h	give this usage help.\n"
+"	-v	show current mixer settings.\n");
+printf(
+"	device	a device to set the level of. %s\n",mixerdev);
+printf("\t\twill allow the levels of these devices to be set:\n\n\t\t");
 
 count=0;
 for(f=0;f<SOUND_MIXER_NRDEVICES;f++)
@@ -245,7 +251,8 @@ puts(
 "  rec | norec	`rec' makes device a recording source, `norec' makes it, well,\n"
 "		not a recording source. :-)\n"
 "\n"
-"If invoked without any args, rexima runs interactively.");
+"If invoked without any args (with the exception of `-d'), rexima runs\n"
+"interactively.");
 }
 
 
@@ -257,15 +264,54 @@ int existmask,canrecmask,isrecmask,stereomask;
 int f,tmp,l,r;
 int found;
 char *ptr;
+int done=0,want_usage=0,want_levels=0;
 
-init(&mixfd,&existmask,&canrecmask,&isrecmask,&stereomask);
+opterr=0;
 
-if(strcmp(argv[1],"-h")==0)
-  usage_help(existmask),exit(1);
+do
+  switch(getopt(argc,argv,"d:hvV"))
+    {
+    case 'd':	/* mixer device */
+      if((mixerdev=malloc(strlen(optarg)+1))==NULL)
+        die("allocate memory");
+      strcpy(mixerdev,optarg);
+      break;
+    case 'h':
+      want_usage=1;
+      break;
+    case 'v': case 'V':	/* show levels */
+      want_levels=1;
+      break;
+    case '?':
+      switch(optopt)
+        {
+        case 'd':
+          fprintf(stderr,"rexima: "
+                  "the `-d' option needs a mixer device to be specified.\n");
+          break;
+        default:
+          fprintf(stderr,"rexima: option `%c' not recognised.\n",optopt);
+        }
+      exit(1);
+    case -1:
+      done=1;
+    }
+while(!done);
 
-/* allow setmixer-style `-V' for this too (miswart) */
-if(strcasecmp(argv[1],"-v")==0)
+
+if(want_usage)
   {
+  init(&mixfd,&existmask,&canrecmask,&isrecmask,&stereomask);
+  
+  usage_help(existmask);
+  
+  exit(0);
+  }
+
+if(want_levels)
+  {
+  init(&mixfd,&existmask,&canrecmask,&isrecmask,&stereomask);
+  
   for(f=0;f<SOUND_MIXER_NRDEVICES;f++)
     if(existmask&(1<<f))
       {
@@ -277,16 +323,24 @@ if(strcasecmp(argv[1],"-v")==0)
         printf("\t[%c]",(isrecmask&(1<<f))?'R':' ');
       putchar('\n');
       }
+  
   exit(0);
   }
 
-/* otherwise, we have 0 or more device, setting pairs to deal with */
-while(argc>1)
+/* if there aren't any "device, setting" pairs, return (without init). */
+if(optind>=argc)
+  return;
+
+/* otherwise, init - we have one or more pairs to deal with. */
+
+init(&mixfd,&existmask,&canrecmask,&isrecmask,&stereomask);
+
+while(optind<argc)
   {
   /* lookup argv[1] in dev_names[] */
   found=0;
   for(f=0;f<SOUND_MIXER_NRDEVICES;f++)
-    if(existmask&(1<<f) && strcmp(argv[1],dev_names[f])==0)
+    if(existmask&(1<<f) && strcmp(argv[optind],dev_names[f])==0)
       {
       found=1;
       break;
@@ -294,53 +348,55 @@ while(argc>1)
   
   if(!found)
     {
-    fprintf(stderr,"rexima: unavailable or unknown device `%s'.\n",argv[1]);
+    fprintf(stderr,"rexima: unavailable or unknown device `%s'.\n",argv[optind]);
     exit(1);
     }
   
-  if(argc<3)
+  if(argc-optind<2)
     {
-    fprintf(stderr,"rexima: no setting specified for device `%s'.\n",argv[1]);
+    fprintf(stderr,"rexima: no setting specified for device `%s'.\n",argv[optind]);
     exit(1);
     }
   
-  argc--; argv++;
+  optind++;
   
   tmp=f;	/* save dev no. */
   
   /* setting can be mono, stereo, or rec/norec.
    * first see if it's rec or norec:
    */
-  if(strcmp(argv[1],"rec")==0 || strcmp(argv[1],"norec")==0)
+  if(strcmp(argv[optind],"rec")==0 || strcmp(argv[optind],"norec")==0)
     {
     if(!(canrecmask&(1<<f)))
       {
-      fprintf(stderr,"rexima: can't set rec/norec on device `%s'.\n",argv[0]);
+      fprintf(stderr,"rexima: can't set rec/norec on device `%s'.\n",
+              argv[optind-1]);
       exit(1);
       }
     
-    mixer_recset(mixfd,tmp,&isrecmask,(argv[1][0]=='r'));
+    mixer_recset(mixfd,tmp,&isrecmask,(argv[optind][0]=='r'));
     }
   else
     {
     int offset_sign=0;
     
     /* see if it's an offset */
-    if(argv[1][0]=='+')
+    if(argv[optind][0]=='+')
       offset_sign=1;
-    if(argv[1][0]=='-')
+    if(argv[optind][0]=='-')
       offset_sign=-1;
 
     /* be sure to puke if level contains funny chars */
-    for(f=(offset_sign?1:0);f<strlen(argv[1]);f++)
-      if(strchr("0123456789, \t",argv[1][f])==NULL)
+    for(f=(offset_sign?1:0);f<strlen(argv[optind]);f++)
+      if(strchr("0123456789, \t",argv[optind][f])==NULL)
         {
-        fprintf(stderr,"rexima: invalid level for device `%s'.\n",argv[0]);
+        fprintf(stderr,"rexima: invalid level for device `%s'.\n",
+                argv[optind-1]);
         exit(1);
         }
     
     /* now check for stereo */
-    if((ptr=strchr(argv[1],','))!=NULL)
+    if((ptr=strchr(argv[optind],','))!=NULL)
       {
       if(offset_sign)
         {
@@ -348,7 +404,7 @@ while(argc>1)
         exit(1);
         }
       
-      l=atoi(argv[1]);
+      l=atoi(argv[optind]);
       r=atoi(ptr+1);
       mixer_setlevel_stereo(mixfd,tmp,l,r);
       }
@@ -357,13 +413,14 @@ while(argc>1)
       /* must be mono; default to 2 for an offset with no size specified. */
       if(offset_sign)
         mixer_change(mixfd,tmp,
-                     (strlen(argv[1])<2)?2*offset_sign:atoi(argv[1]));
+                     (strlen(argv[optind])<2)?
+                     2*offset_sign:atoi(argv[optind]));
       else
-        mixer_setlevel(mixfd,tmp,atoi(argv[1]));
+        mixer_setlevel(mixfd,tmp,atoi(argv[optind]));
       }
     }
     
-  argc--; argv++;
+  optind++;
   }
 
 exit(0);
@@ -382,13 +439,14 @@ int existmask,canrecmask,isrecmask,stereomask;
 int firstdev,lastdev;
 int f;
 
-if(argc>1) cmdline_main(argc,argv);	/* doesn't return */
+if(argc>1)
+  cmdline_main(argc,argv);  /* returns (without init) if we're interactive */
 
 init(&mixfd,&existmask,&canrecmask,&isrecmask,&stereomask);
 
 /* later things assume existmask is non-zero, so... */
 if(!existmask)
-  fprintf(stderr,"rexima: /dev/mixer has no devices!\n"),exit(1);
+  fprintf(stderr,"rexima: mixer has no devices!\n"),exit(1);
 
 init_term();
 setupframe(existmask,canrecmask,&firstdev,&lastdev);
